@@ -1,6 +1,6 @@
 /*
  * Photos - access, organize and share your photos on GNOME
- * Copyright © 2015, 2016 Red Hat, Inc.
+ * Copyright © 2015 – 2016 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -667,7 +667,7 @@ photos_operation_insta_curve_prepare (GeglOperation *operation)
       break;
 
     case PHOTOS_OPERATION_INSTA_PRESET_NONE:
-    case PHOTOS_OPERATION_INSTA_PRESET_GRAY:
+    case PHOTOS_OPERATION_INSTA_PRESET_HEFE:
     default:
       g_assert_not_reached ();
     }
