@@ -142,6 +142,17 @@ photos_tracker_change_monitor_add_event (PhotosTrackerChangeMonitor *self, Photo
 }
 
 
+static void
+photos_tracker_change_monitor_remove_timeout (PhotosTrackerChangeMonitor *self)
+{
+  if (self->pending_events_id != 0)
+    {
+      g_source_remove (self->pending_events_id);
+      self->pending_events_id = 0;
+    }
+}
+
+
 static void
 photos_tracker_change_monitor_send_events (PhotosTrackerChangeMonitor *self, GHashTable *id_table, GQueue *events)
 {
@@ -285,8 +296,7 @@ photos_tracker_change_monitor_add_pending_event (PhotosTrackerChangeMonitor *sel
 {
   PhotosTrackerChangeEvent *change_event;
 
-  if (self->pending_events_id != 0)
-    g_source_remove (self->pending_events_id);
+  photos_tracker_change_monitor_remove_timeout (self);
 
   g_hash_table_insert (self->unresolved_ids, GINT_TO_POINTER (event->subject), NULL);
   g_hash_table_insert (self->unresolved_ids, GINT_TO_POINTER (event->predicate), NULL);
@@ -297,11 +307,9 @@ photos_tracker_change_monitor_add_pending_event (PhotosTrackerChangeMonitor *sel
   if (self->pending_events->length >= CHANGE_MONITOR_MAX_ITEMS)
     photos_tracker_change_monitor_process_events (self);
   else
-    self->pending_events_id = g_timeout_add_full (G_PRIORITY_DEFAULT,
-                                                  CHANGE_MONITOR_TIMEOUT,
-                                                  (GSourceFunc) photos_tracker_change_monitor_process_events,
-                                                  g_object_ref (self),
-                                                  g_object_unref);
+    self->pending_events_id = g_timeout_add (CHANGE_MONITOR_TIMEOUT,
+                                             (GSourceFunc) photos_tracker_change_monitor_process_events,
+                                             self);
 }
 
 
@@ -356,6 +364,8 @@ photos_tracker_change_monitor_dispose (GObject *object)
 {
   PhotosTrackerChangeMonitor *self = PHOTOS_TRACKER_CHANGE_MONITOR (object);
 
+  photos_tracker_change_monitor_remove_timeout (self);
+
   g_clear_object (&self->queue);
   g_clear_object (&self->resource_service);
 
